function [Mean]= MeanMTN(Trunc,mu,Sigma,n)

% MeanMTN: Simulate the expetted value of a truncated multivariate Normal, when only
% values above the thershold are allowed
%*****************************************************************
%INPUTS:
% Trunc,         truncation points
% mu,            expected value of the Normal distribution
% Sigma,         Covariance matrix of the Normal distribution
% n,             number of samples

%OUTPUTS:
% Mean,          simulated expected value of the truncated normal distribution

%USAGE:
%[Mean]= MeanMTN(Trunc,mu,Sigma,n)

%
%Copyright (c) 2010 Francesco C Stingo.
%*****************************************************************

ki = length(mu);
pos = 0;
while(isempty(find(pos)))
    Xs = mvnrnd(mu,Sigma,n);
    pos =ones(n,1);
    for i=1:ki
        pos= pos.*(Xs(:,i)>Trunc(i));
    end
    pos=pos+0;
    Xs = Xs(find(pos),:);
    Mean = mean(Xs,1)';
end
