function [R2] = PostEV(MargGam,threshold,q,Y,X,SigMmean,alph)

%PostEV: Posterior Expected value of the regression coefficients
%*****************************************************************
%INPUTS:
% MargGam,      matrix of the marginal posterior probabilities of inclusion of the arrows
% threshold,    posterior probaility threshold
% Y,            response variables (targets), matrix (N by G)
% X,            independent variables (miRNAs), matrix (N by M)
% alph,         hyperparameter of the Gamma prior on the regression
%               coefficients (hyperparameter $c$ in the manuscript)

%OUTPUTS:
% R2,           $R^2$ index

%USAGE:
%[R2] = PostEV(MargGam,threshold,q,Y,X,SigMmean)

%
%NOTES:
% Functions called: MeanMTN
%
%Copyright (c) 2010 Francesco C Stingo.
%*****************************************************************

[nOss p]=size(X);
GammaVett = MargGam>threshold;
res = cell(q,1);
MeanBeta=cell(q,1);
res1 = []; sum1 = 0;  sum2 = 0;
bi = 5000; sig = mean(SigMmean(bi:end,:));
R2i = zeros(q,1); Narrows = zeros(q,1);
varRid = zeros(q,1); resRid = zeros(q,1);
res1 = []; nsim = 10000000;

for i=1:q
    gammaYi = GammaVett((((i-1)*p)+1):(i*p));
    ki = sum(gammaYi);           
    Yi = Y(:,i);
    if ki==0
        res{i}=Yi;
        res1 = [res1  Yi'];
        EbetaV = [];
        elseif ki==1
            Zi = X(:,find(gammaYi));   
            V = inv(Zi'*Zi);
            C = -(Yi'*Zi)'-(sqrt(sig(i))/alph);
            mm = V'*C;
            vv = sig(i)*V;
            if ((normpdf(-mm/sqrt(vv))*sqrt(vv))/(1-normcdf(-mm/sqrt(vv))))==Inf
                EbetaV = 0;             
            else
                EbetaV = mm +(normpdf(-mm/sqrt(vv))/(1-normcdf(-mm/sqrt(vv))))*sqrt(vv); 
            end
            MeanBeta{i} = EbetaV;
            res{i} = Yi + Zi*MeanBeta{i}; 
            res1 = [res1  (Yi + Zi*MeanBeta{i})'];
    else
        Zi = X(:,find(gammaYi));   
        V = inv(Zi'*Zi);
        C = -(Yi'*Zi)'-(sqrt(sig(i))/alph).*ones(ki,1);
        mm = V'*C;
        vv = sig(i).*V;        
        Sigma = vv; mu = mm; Trunc = zeros(ki,1)';
        EbetaV = MeanMTN(Trunc,mu,Sigma,nsim);                           
        MeanBeta{i} = EbetaV;
        res{i} = Yi + Zi*MeanBeta{i}; 
        res1 = [res1  (Yi + Zi*MeanBeta{i})'];
    end
    sum1 = sum1 + Yi'*Yi;
    sum2 = sum2 + res{i}'*res{i};
    varRid(i) = Yi'*Yi;
    resRid(i) = res{i}'*res{i};
    R2i(i) = 1-(res{i}'*res{i})/(Yi'*Yi);
    Narrows(i) = ki;
end
R2 = 1-sum2/sum1;

