function[Uni,UniD,GammaD,logProb,logProbD,AD,S,accepttau,tauSt,accepttau2,tau2St,accepttau3,tau3St,accepttau4,tau4St,accepttau5,tau5St,...
    acceptgam,acceptsig,FreqMat,MargGam,SigMmean]= ... 
bvsme_miRNA_Fbn(gamprec,X,Y,e,a_sig,b_sig,alph,nmetr,r,pPar, SPicTar, SMir,Starget1,Starget2,Spita,eta,...
a_tau,b_tau,taupropsd,tauprec,tauprec2,tauprec3,tauprec4,tauprec5,sigVprec)

%bvsme_miRNA: Bayesian Variable Selection with Metropolis search
%*****************************************************************
%INPUTS:
% gamprec,      starting binary vector.
% X,            independent variables (miRNAs), matrix (N by M)
% Y,            response variables (targets), matrix (N by G)
% e,			variance of the proposal distribution for the parameters $\sigma_g$ 
% a_sig,b_sig,  hyperparameter of inverse-Gamma priors on $\sigma_g$ 
% alph,         hyperparameter of the Gamma prior on the regression coefficients (hyperparameter $c$ in the manuscript)
% nmetr,        total number of iterations (multiple of 1000)
% r,            initial number of included variables
% pPar,         probability of add/delete for Metropolis(suggested 1/2)
% SPicTar,      Vector of PicTar Scores (G*M) by 1
% SMir,         Vector of miRanda Scores (G*M) by 1
% Starget1,     Vector of aggregate Target Scan Scores (G*M) by 1
% Starget2,     Vector of total Target Scan Scores (G*M) by 1
% Spita,		Vector of PITA Scores (G*M) by 1
% eta,          hyperparamete of the data integration prior (5)($\eta$)
% a_tau, b_tau, Hyperparameters of the gamma prior on tau (hyperparameters $a_{\tau}$ and $b_{\tau}$ in the manuscript)
% taupropsd,    Standard Deviation of the Random Walk Proposal for updating tau (step 2 of the MCMC algorithm)
% tauprec-		Starting values for $\tau_1$-$\tau_5$  
% tauprec5,	 
% sigVprec,     Starting values for $\sigma_g$s

%OUTPUTS:
% Uni,  	  	vector with number of ones of visited models
% UniD,         vector with number of ones of distinct visited models
% GammaD,   	distinct visited models, ordered according to    their (normalized) relative posterior probability (cell array of strings)
% logProb,      log-relative posterior probabilities of all visited models
% logProbD,     log-relative posterior probabilities of distinct visited models
% PostGamD,  	normalized ordered relative probabilities of distinct visited models 
% AD, S,    	iteration numbers of accepted additions/deletions (AD) and swaps (S)
% accepttau-  	acceptance ratio for the metropolis proposal for $\tau$s 
% accepttau5,
% tauSt-     	sampled values of $\tau$s
% tauSt5,
% acceptgam, 	acceptance ratio for the metropolis proposal for model 
% acceptsig,	acceptance ratio for the metropolis proposals for $\sigma_g$s 
% FreqMat,		matrix of the empirical frequencies of arrows inclusion during the MCMC
% MargGam,		vector of the marginal posterior probabilities of inclusion of the arrows
% SigMmean		vector of the posterior means of $\sigma_g$s

%USAGE:
%[Uni,UniD,GammaD,logProb,logProbD,AD,S,accepttau,tauSt, accepttau2,tau2St,accepttau3,tau3St,accepttau4,tau4St,accepttau5,tau5St,...
%acceptgam,acceptsig,FreqMat,MargGam,SigMmean]= ... 
% bvsme_miRNA_Fbn(gamprec,X,Y,e,a_sig,b_sig,alph,nmetr,r,pPar, SPicTar, SMir,Starget1,Starget2,Spita,eta,...
%a_tau,b_tau,taupropsd,tauprec,tauprec2,tauprec3,tauprec4, tauprec5,sigVprec)

%
%NOTES:
% Data must be centered.
% Scores (PicTar, miRanda, ecc...) must be positive and normalized. If the number of integrated scores is not 5 the code must be (easely) modified 
% GammaD is in cell array format (elements are strings)
% Search for replicates among visited accepted models of same size
% Set the variable "bi" to decide the number of burn-in iterations
% Functions called: metro_miRNA_Fbn
%REFERENCE: 
%
%Copyright (c) 2010 Francesco C Stingo.
%*****************************************************************

[nOss p]=size(X);
[q]=size(Y,2); 		% p independent variables, q responses, nOss observations

if isempty(gamprec)
    randp=randperm(p*q);
    gamprec=zeros(1,p*q);
    onenum=randp(1:r);
    gamprec(onenum)=1;
else
end

disp(' ')
disp('------- RUNNING METROPOLIS ...')
disp(' ')

nPar=100;
Gamma{1}=num2str(find(gamprec)); GammaD{1}=num2str(find(gamprec));
Uni(1)=r; UniD(1)=r; logProb=zeros(1,nmetr+1);
AD=[]; S=[]; conta=1; contaD=1;

accepttau = 0; accepttau2 = 0;
accepttau3 = 0; accepttau4 = 0;
accepttau5 = 0; acceptgam = 0; acceptsig = 0;


%---- Data integration prior (using tau - tau5)

w = exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)); 


%---- Log-prior for tau-tau5

loglik = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
    +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;
for iter = 1:(p*q)
    if gamprec(iter)==1
        loglik = loglik + log(w(iter)); 
     else
        if w(iter)==1
  	    w(iter)=1-10^-16;
        else
        end
        loglik = loglik + log(1.-w(iter)); 
    end        
end


% ---- calculate the marginal log-likelihood

VettMML=zeros(1,q);
for i=1:q
    gammaYi=gamprec((((i-1)*p)+1):(i*p));
    ki=sum(gammaYi);  
    Yi=Y(:,i);
    sig = sigVprec(i); %sig  la varianza (\sigma^2)
    if ki>0        
        Zi=X(:,find(gammaYi));        
        Ci = -Yi'*Zi-(sqrt(sig)/alph)*ones(1,ki);
        Vi = inv(Zi'*Zi);
        rr = -((nOss-ki)/2)*log(2*pi)-(nOss/2)*log(sig)-ki*log(alph)-(1/(2*sig))*(Yi'*Yi-Ci*Vi*Ci')+...
            0.5*log(det(Vi))+log(mvncdf(zeros(ki,1),-Vi'*Ci',sig.*Vi)); 
    else
        rr = log(mvnpdf(Yi,zeros(nOss,1),diag(ones(nOss,1).*sig)));
    end
    VettMML(i)= rr;           
end      

% ----calculate the log-prior for the selection indicators

VettPgamma=zeros(1,p*q);
for i=1:(p*q)
    if w(i)==1
        w(i)=1-10^-16;
    else
    end
    ss = gamprec(i)*log(w(i))+(1-gamprec(i))*log(1-w(i));
    VettPgamma (i)= ss;
end


% ----calculate the log-prior for the $\sigma_g$s

VettSig=zeros(1,q);
for i=1:q
    sig = sigVprec(i);
    gammaYi=gamprec((((i-1)*p)+1):(i*p));
    ki=sum(gammaYi);
    a_sigNew = (a_sig+ki)/2;
    VettSig(i) = log((1/gamma(a_sigNew))*(b_sig^a_sigNew)*sig^(-a_sigNew-1)*exp(-b_sig/sig));    
end


tic;
tauSt = zeros(1,nmetr); tau2St = zeros(1,nmetr);
tau3St = zeros(1,nmetr); tau4St = zeros(1,nmetr);
tau5St = zeros(1,nmetr);
Uni = zeros(1,nmetr); SigMmean = zeros(nmetr/nPar,q);

probprec=sum(VettMML)+sum(VettPgamma);
logProb(conta)= probprec+sum(VettSig);
logProbD(contaD)=probprec+sum(VettSig);

bi = 500000; % Number of burn-in iterations 

freTot = zeros(p*q,1);
lenMar = p*q;
MargGam=zeros(1,lenMar);
TotProb =0;

while conta<nmetr+1
    
%-------------------- Metropolis (nPar iterations)    
    [UniSt,ProbSt,GammaSt,storeAD,storeS,tauSt1,tau2St1,tau3St1,tau4St1,tau5St1,accepttau,accepttau2,accepttau3,accepttau4,accepttau5,w,loglik,tauprec,tauprec2,...
      tauprec3,tauprec4,tauprec5,acceptgam,acceptsig,VettMML,VettPgamma,SigMean,sigVprec,probprec,VettSig]=...
        metro_miRNA_Fbn(gamprec,probprec,X,Y,e,a_sig,b_sig,alph,w,nPar,pPar,SPicTar,SMir,Starget1,Starget2,Spita,tauprec,tauprec2,...
      tauprec3,tauprec4,tauprec5,loglik,accepttau,accepttau2,accepttau3,accepttau4,accepttau5,a_tau,b_tau,eta,taupropsd,acceptgam,acceptsig,VettMML,VettPgamma,VettSig,sigVprec);

tauSt(conta+1:conta+nPar) = tauSt1; 
tau2St(conta+1:conta+nPar) = tau2St1;
tau3St(conta+1:conta+nPar) = tau3St1; 
tau4St(conta+1:conta+nPar) = tau4St1;
tau5St(conta+1:conta+nPar) = tau5St1;

logProb(conta+1:conta+nPar)=ProbSt; 
Uni(conta+1:conta+nPar) = UniSt;
indiciD=sort([storeAD storeS]); 
GammaStD = GammaSt(indiciD); 
logProbD=[logProbD, ProbSt(indiciD)]; 
ProbD = ProbSt(indiciD); UniD=[UniD, UniSt(indiciD)];
AD=cat(2, AD, (conta-1)+storeAD);  S=cat(2, S, (conta-1)+storeS);
conta=conta+nPar; SigMmean((conta-1)/nPar,:) = SigMean;
contaD=length(logProbD); gamprec=zeros(1,p*q);
gamprec(str2num(GammaSt{nPar}))=ones(1,length(str2num(GammaSt{nPar})));     

  
%--- Calculate the posterior probabilities

    if (conta>bi & size(ProbD)>0)

        %----------- Empirical Frequencies

        GammaBI=GammaStD;
        a = size(GammaBI);        
        FreqUni = [];
        for i=1:a
            FreqUni = [FreqUni str2num(GammaBI{i})];
        end
        fre = tabulate(FreqUni);
        fsize = size(fre);
        m = p*q - fsize(1);
        Z = zeros(m,1);
        fre = [fre(:,2);Z];
        freTot = fre+freTot;


     	  %----------- Marginal Frequencies

        postval = ProbD;%(1:sss(1));
        TotProb = TotProb+sum(postval);
        for i=1:length(GammaBI)
            numgami=str2num(GammaBI{i});
                if ~isempty(numgami)
                    MargGam(numgami)=MargGam(numgami)+postval(i)*ones(1,length(numgami));
                else
                end
        end
    end
    
disp(sprintf('Iter.: %3d, Time: %3.3fs, acpttau: %1.3f, acptgam: %1.3f, acptsig: %1.3f, Uni: %3d, Tau: %1.3f,Tau2: %1.3f',conta-1,toc,(accepttau/(conta-1)),(acceptgam/(conta-1)),(acceptsig/(q*(conta-1)/nPar)),Uni(conta-1),tauSt(conta-1),tau2St(conta-1))); tic;

end

% ------ Calculate acceptance ratios

accepttau = accepttau/nmetr; accepttau2 = accepttau2/nmetr;
accepttau3 = accepttau3/nmetr; accepttau4 = accepttau4/nmetr;
accepttau5 = accepttau5/nmetr;
acceptgam = acceptgam/nmetr;
acceptsig = acceptsig/(q*nmetr/nPar);

MargGam = MargGam./ TotProb;

%------- Reframing the posterior probabilities in a matrix format
FreqMat = [];
for i=1:q
    FreqMat = [FreqMat ; freTot(1+p*(i-1):p*i)'];
end


disp(' ')
disp('--- End.') 
