function [Uni,UniD,GammaD,logProb,logProbD,AD,S,accepttau,tauSt,accepttau2,tau2St,accepttau3,tau3St,accepttau4,tau4St,accepttau5,tau5St,...
    acceptgam,acceptsig,FreqMat,MargGam,SigMmean]= ...
mainprog_Fbn(gamprec,X,Y,e,a_sig,b_sig,alph,nmetr,r,pPar,SPicTar,SMir,Starget1,Starget2,Spita,eta,a_tau,b_tau,taupropsd,...
tauprec,tauprec2,tauprec3,tauprec4,tauprec5,sigVprec);

%*****************************************************************
%INPUTS:
% gamprec,      starting binary vector.
% X,            independent variables (miRNAs), matrix (N by M)
% Y,            response variables (targets), matrix (N by G)
% e,			variance of the proposal distribution for the parameters $\sigma_g$ 
% a_sig,b_sig,  hyperparameter of inverse-Gamma priors on $\sigma_g$ 
% alph,		hyperparameter of the Gamma prior on the regression coefficients (hyperparameter $c$ in the manuscript)
% nmetr,        total number of iterations (multiple of 1000)
% r,            initial number of included variables
% pPar,         probability of add/delete for Metropolis(suggested 1/2)
% SPicTar,      Vector of PicTar Scores (G*M) by 1
% SMir,		Vector of miRanda Scores (G*M) by 1
% Starget1,	Vector of aggregate Target Scan Scores (G*M) by 1
% Starget2,	Vector of total Target Scan Scores (G*M) by 1
% Spita,		Vector of PITA Scores (G*M) by 1
% eta,          hyperparamete of the data integration prior (5)($\eta$)
% a_tau, b_tau, Hyperparameters of the gamma prior on tau (hyperparameters $a_{\tau}$ and $b_{\tau}$ in the manuscript)
% taupropsd,    Standard Deviation of the Random Walk Proposal for updating tau (step 2 of the MCMC algorithm)
% tauprec-		Starting values for $\tau_1$-$\tau_5$  
% tauprec5,	 
% sigVprec,	Starting values for $\sigma_g$s

%OUTPUTS:
% Uni,  	  	vector with number of ones of visited models
% UniD,    	vector with number of ones of distinct visited models
% GammaD,   	distinct visited models, ordered according to    their (normalized) relative posterior probability (cell array of strings)
% logProb,      log-relative posterior probabilities of all visited models
% logProbD,     log-relative posterior probabilities of distinct visited models
% AD, S,    	iteration numbers of accepted additions/deletions (AD) and swaps (S)
% accepttau-  	acceptance ratio for the metropolis proposal for $\tau$s 
% accepttau5,
% tauSt-     	sampled values of $\tau$s
% tauSt5,
% acceptgam, 	acceptance ratio for the metropolis proposal for model 
% acceptsig,	acceptance ratio for the metropolis proposals for $\sigma_g$s 
% FreqMat,		matrix of the empirical frequencies of arrows inclusion during the MCMC
% MargGam,		matrix of the marginal posterior probabilities of  inclusion of the arrows
% SigMmean		vector of the posterior means of $\sigma_g$s

%USAGE:
%[Uni,UniD,GammaD,logProb,logProbD,AD,S,accepttau,tauSt, accepttau2,tau2St,accepttau3,tau3St,accepttau4,tau4St,accepttau5,tau5St,...
%    acceptgam,acceptsig,FreqMat,MargGam,SigMmean]= ...
%mainprog_Fbn(gamprec,X,Y,e,a_sig,b_sig,alph,nmetr,r,pPar,SPicTar,SMir,Starget1,Starget2,Spita,eta,a_tau,b_tau,taupropsd,...
%tauprec,tauprec2,tauprec3,tauprec4,tauprec5,sigVprec);

%NOTES:
% Data must be centered. 
% GammaD is in cell array format (elements are strings)
% Functions called: bvsme_miRNA_Fbn
%
%REFERENCE: 
%
%Copyright (c) 
%*****************************************************************

[n p] = size(X); % number of time-condition-replicate, and number of miRNAs respectively
q = size(Y,2); % number of genes

UniY=[];
UniSPicTar = [];
for iter = 1:q
    UniSPicTar = [UniSPicTar SPicTar(iter,:)]; 
end

UniSMir = [];
for iter = 1:q
    UniSMir = [UniSMir SMir(iter,:)]; 
end

UniStarget1 = [];
for iter = 1:q
    UniStarget1 = [UniStarget1 Starget1(iter,:)]; 
end
UniStarget2 = [];
for iter = 1:q
    UniStarget2 = [UniStarget2 Starget2(iter,:)]; 
end
UniSpita = [];
for iter = 1:q
    UniSpita = [UniSpita Spita(iter,:)]; 
end

%--- Centering the X matrix
for i=1:p
    X(:,i)=X(:,i)-mean(X(:,i));
end

disp(' ')
disp('------- Calling the variable selection code now  --------')
disp(' ')


[Uni,UniD,GammaD,logProb,logProbD,AD,S,accepttau,tauSt,accepttau2,tau2St,accepttau3,tau3St,accepttau4,tau4St,accepttau5,tau5St,...
    acceptgam,acceptsig,FreqMat,MargGam,SigMmean]= ... 
bvsme_miRNA_Fbn(gamprec,X,Y,e,a_sig,b_sig,alph,nmetr,r,pPar,UniSPicTar,UniSMir,UniStarget1,UniStarget2,UniSpita,eta,...
a_tau,b_tau,taupropsd,tauprec,tauprec2,tauprec3,tauprec4,tauprec5,sigVprec);
