function[UniSt,ProbSt,GammaSt,storeAD,storeS,tauSt,tau2St,tau3St,tau4St,tau5St,accepttau,accepttau2,accepttau3,accepttau4,accepttau5,w,loglik,tauprec,tauprec2,...  
    tauprec3,tauprec4,tauprec5,acceptgam,acceptsig,VettMML,VettPgamma,SigMean,sigVprec,probprec,VettSig]=...
    metro_miRNA_Fbn(gamprec,probprec,X,Y,e,a_sig,b_sig,alph,w,nPar,pPar,SPicTar,SMir,Starget1,Starget2,Spita,tauprec,tauprec2,...    
    tauprec3,tauprec4,tauprec5,loglik,accepttau,accepttau2,accepttau3,accepttau4,accepttau5,a_tau,b_tau,eta,taupropsd,acceptgam,acceptsig,VettMML,VettPgamma,VettSig,sigVprec);

%*****************************************************************
%INPUTS:
% gamprec,      starting binary vector.
% probprec,   	log-relative posterior probability of gamprec
% X,            independent variables (miRNAs), matrix (N by M)
% Y,            response variables (targets), matrix (N by G)
% e,			variance of the proposal distribution for the parameters $\sigma_g$ 
% a_sig,b_sig,  hyperparameter of inverse-Gamma priors on $\sigma_g$ 
% alph,		hyperparameter of the Gamma prior on the regression coefficients (hyperparameter $c$ in the manuscript)
% w,          	hyperparameter (Bernoulli priors)
% nPar,       	number of iterations
% pPar,         probability of add/delete for Metropolis(suggested 1/2)
% SPicTar,      Vector of PicTar Scores (G*M) by 1
% SMir,		Vector of miRanda Scores (G*M) by 1
% Starget1,	Vector of aggregate Target Scan Scores (G*M) by 1
% Starget2,	Vector of total Target Scan Scores (G*M) by 1
% Spita,		Vector of PITA Scores (G*M) by 1
% tauprec-		Current values for $\tau_1$-$\tau_5$  
% tauprec5,	 
% loglik,     	Log-prior for tau-tau5
% accepttau-  	acceptance ratio for the metropolis proposal for $\tau$s 
% accepttau5,
% a_tau, b_tau, Hyperparameters of the gamma prior on tau (hyperparameters $a_{\tau}$ and $b_{\tau}$ in the manuscript)
% eta,          hyperparamete of the data integration prior (5)($\eta$)
% taupropsd,    Standard Deviation of the Random Walk Proposal for updating tau (step 2 of the MCMC algorithm)
% acceptgam, 	acceptance ratio for the metropolis proposal for model 
% acceptsig,	acceptance ratio for the metropolis proposals for $\sigma_g$s 
% VettMML,     	vector marginal likelihood (q by 1)
% VettPgamma,  	vector of prior probability of R given tau (p*q by 1)
% VettSig,		vector of priors on $\sigma_g$ evaluated at the current values
% sigVprec,	vector of the sampled values $\sigma_g$s

%OUTPUTS:
% UniSt,  	  	vector with number of ones of visited models
% ProbSt, 	  	log-relative posterior probabilities of visited models
% GammaSt,   	cell with positions of 1's in visited vectors
% storeAD, 	iteration numbers of accepted additions/deletions (AD) and swaps (S)	
% storeS,    	
% tauSt-     	vectors of the sampled values of $\tau$s
% tauSt5,
% accepttau-  	acceptance ratio for the metropolis proposal for $\tau$s 
% accepttau5,
% w,          	hyperparameter (Bernoulli priors)
% loglik,		Log-prior for tau-tau5
% tauprec-		Current values for $\tau_1$-$\tau_5$  
% tauprec5,	 
% acceptgam, 	acceptance ratio for the metropolis proposal for model 
% acceptsig,	acceptance ratio for the metropolis proposals for $\sigma_g$s 
% VettMML,     	vector marginal likelihood (q by 1)
% VettPgamma,  	vector of prior probability of R given tau (p*q by 1)
% SigMean,		matrix of the sampled $\sigma_g$s
% sigVprec,	vector of the sampled values $\sigma_g$s
% probprec, 	log-relative posterior probability of the last accepted gamprec
% VettSig,		vector of priors on $\sigma_g$ evaluated at the current values

%USAGE:
%[UniSt,ProbSt,GammaSt,storeAD,storeS,tauSt,tau2St,tau3St,tau4St, tau5St,accepttau,accepttau2,accepttau3,accepttau4,accepttau5,w, loglik,tauprec,tauprec2,...   
%tauprec3,tauprec4,tauprec5,acceptgam,acceptsig,VettMML, VettPgamma,SigMean,sigVprec,probprec,VettSig]=...
% metro_miRNA_Fbn(gamprec,probprec,X,Y,e,a_sig,b_sig,alph,w,nPar,pPar,SPicTar, SMir,Starget1,Starget2,Spita,tauprec,tauprec2,...      
%tauprec3,tauprec4,tauprec5,loglik,accepttau,accepttau2, accepttau3,accepttau4,accepttau5,a_tau,b_tau,eta,taupropsd,acceptgam,acceptsig,VettMML,VettPgamma,VettSig,sigVprec);       
%
%NOTES:
% Called by bvsme_miRNA_Fbn
% Functions called: mml_miRNAbn
% An adding move is forced with empty model
% A delete move is forced with full model

%REFERENCE:     
%Copyright (c) .
%*****************************************************************


[nOss p]=size(X); [q]=size(Y,2); storeAD=[]; storeS=[];
GammaSt=cell(nPar,1); ProbSt=zeros(1,nPar); UniSt=zeros(1,nPar);
SigM=zeros(1,q); cost = 0.000001;
tauSt=zeros(1,nPar); tau2St=zeros(1,nPar); tau3St=zeros(1,nPar); 
tau4St=zeros(1,nPar); tau5St=zeros(1,nPar); 


for i=1:nPar
    gamnew=gamprec;
    poszeros=find(gamprec==0);
    VettMMLnew=VettMML;
    VettPgammaNew=VettPgamma;
    elem2=[];
    if length(poszeros)==(p*q)   % Forced to add variable
        chi='AD';
        choice=ceil(rand(1)*(p*q));
        if choice==0
            choice=1;
        else
        end
        gamnew(choice)=1;
        [rr,s1,s2,ind] = mml_miRNAbn(gamnew,X,Y,choice,elem2,alph,w,sigVprec);
        VettMMLnew(ind)=rr; 
        VettPgammaNew(choice)=s1;
    elseif  isempty(poszeros)          % Forced to delete variable
        chi='AD';
        choice=ceil(rand(1)*(p*q));
        if choice==0
            choice=1;
        else
        end
        gamnew(choice)=0;
        [rr,s1,s2,ind] = mml_miRNAbn(gamnew,X,Y,choice,elem2,alph,w,sigVprec);                  
        VettMMLnew(ind)=rr; 
        VettPgammaNew(choice)=s1 ;      
    else            
        x=rand(1);
        if x<pPar
%-------------------Adding/Deleting
            chi='AD';
            choice=ceil(rand(1)*(p*q));
            if choice==0
            choice=1;
            else
            end
            gamnew(choice)=abs(gamnew(choice)-1);
            [rr,s1,s2,ind] = mml_miRNAbn(gamnew,X,Y,choice,elem2,alph,w,sigVprec);                             
            VettMMLnew(ind)=rr; 
            VettPgammaNew(choice)=s1 ;    
        else
%-------------------Swapping
            chi='S';
            poszeros=find(gamprec==0);
            posones=find(gamprec==1);
            choice1=ceil(rand(1)*length(poszeros));
            if choice1==0
                choice1=1;
            else
            end
            choice2=ceil(rand(1)*length(posones));
            if choice1==0
                choice1=1;
            else
            end
            elem1=poszeros(choice1); %positiom of the element of gamma set to 1
            elem2=posones(choice2); %positiom of the element of gamma set to 0
            gamnew(elem1)=1;            
            gamnew(elem2)=0;
            if abs(elem1-elem2)<p
                [rr,s1,s2,ind] = mml_miRNAbn(gamnew,X,Y,elem1,elem2,alph,w,sigVprec);                                 
                VettMMLnew(ind)=rr; 
                VettPgammaNew(choice1)=s1 ;
                VettPgammaNew(choice2)=s2 ;
            else        
                [rr,s1,s2,ind] = mml_miRNAbn(gamnew,X,Y,elem1,elem2,alph,w,sigVprec);
                VettMMLnew(ind)=rr; 
                VettPgammaNew(choice1)=s1 ;
                VettPgammaNew(choice2)=s2 ;    
                [rr,s1,s2,ind] = mml_miRNAbn(gamnew,X,Y,elem2,elem1,alph,w,sigVprec);
                VettMMLnew(ind)=rr;
            end
        end
    end
          
    probnew=sum(VettMMLnew)+sum(VettPgammaNew);
    move=min(0, (probnew-probprec) );
    if move==0
        eval(['store',chi,'=[store',chi,' i];'])
        probprec=probnew;
        gamprec=gamnew;     
        GammaSt{i}=num2str(find(gamnew));
        UniSt(i)=length(find(gamnew)); 
       	acceptgam=acceptgam+1;
        VettMML = VettMMLnew;
        VettPgamma = VettPgammaNew;
%---- Update of the conditional likelihood of tau as well   
        
	   w = exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)); 
        
	   loglik = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
        +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;
        
	   for (iter = 1:(p*q))
             if(gamprec(iter)==1) 
                loglik = loglik + log(w(iter)); 
             else
                if w(iter)==1
	            w(iter)=1-10^-16;
                else
                end
                loglik = loglik + log(1.-w(iter)); 
             end
        end 
    else if (rand(1) < exp(move))
            eval(['store',chi,'=[store',chi,' i];'])
            probprec=probnew;
            gamprec=gamnew;
            GammaSt{i}=num2str(find(gamnew));       
            UniSt(i)=length(find(gamnew));
            acceptgam=acceptgam+1;
            VettMML = VettMMLnew;
            VettPgamma = VettPgammaNew;


%---- Update of the conditional likelihood of tau as well               

            w = exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)); 
            loglik = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
            +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;
            for (iter = 1:(p*q))
                 if(gamprec(iter)==1) 
                    loglik = loglik + log(w(iter)); 
                 else
                    if w(iter)==1
	                w(iter)=1-10^-16;
                    else
                    end
                    loglik = loglik + log(1.-w(iter)); 
                 end
            end
        else
            GammaSt{i}=num2str(find(gamprec));
            UniSt(i)=length(find(gamprec));           
        end
    end

%-------------------- Update tau-tau5
%-------------------- tau ---------------
    atap=0;
    while atap==0
        tauprop = normrnd(tauprec,taupropsd);
        if tauprop > 0
            atap=1;
        end
    end
    if tauprop > 0        
       wprop = exp(eta+tauprop*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprop*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)); 
       loglik_prop = (a_tau - 1)*log(tauprop) - tauprop/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
       +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;
     for (iter = 1:(p*q))
         if(gamprec(iter)==1) 
            loglik_prop = loglik_prop + log(wprop(iter)); 
            VettPgammaNew(iter) = log(wprop(iter));
         else
            if wprop(iter)==1
                wprop(iter) = 1-10^-16;
            else
            end
            loglik_prop = loglik_prop + log(1.-wprop(iter)); 
            VettPgammaNew(iter) = log(1.-wprop(iter)); 
          end
       end        
       if(((loglik_prop+log(1-normcdf(0,tauprec,taupropsd)))-(loglik+log(1-normcdf(0,tauprop,taupropsd)))) > log(unifrnd(0,1,1,1)))
          tauprec = tauprop;
          w = wprop;
          loglik = loglik_prop;
          accepttau = accepttau + 1;
          VettPgamma = VettPgammaNew;          
%---- Update of the conditional likelihood of gamma as well
          probprec = sum(VettMML) + sum(VettPgamma);
       else
       end
    else
    end
    tauSt(i) = tauprec;
    
%-------------------- tau2 ---------------
    atap=0;
    while atap==0
        tauprop2 = normrnd(tauprec2,taupropsd);
        if tauprop2 > 0
            atap=1;
        end
    end
    if tauprop2 > 0        
       %wprop = exp(eta+tauprop*SPicTar)./(1.+exp(eta+tauprop*SPicTar)); 
       wprop = exp(eta+tauprec*SPicTar+tauprop2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprop2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprec5*Spita)); 
       loglik_prop = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprop2) - tauprop2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
       +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;
       for (iter = 1:(p*q))
         if(gamprec(iter)==1) 
            loglik_prop = loglik_prop + log(wprop(iter)); 
            VettPgammaNew(iter) = log(wprop(iter));
         else
            if wprop(iter)==1
                wprop(iter) = 1-10^-16;
            else
            end
            loglik_prop = loglik_prop + log(1.-wprop(iter)); 
            VettPgammaNew(iter) = log(1.-wprop(iter)); 
          end
       end        
       if(((loglik_prop+log(1-normcdf(0,tauprec2,taupropsd)))-(loglik+log(1-normcdf(0,tauprop2,taupropsd)))) > log(unifrnd(0,1,1,1)))
          tauprec2 = tauprop2;
          w = wprop;
          loglik = loglik_prop;
          accepttau2 = accepttau2 + 1;
          VettPgamma = VettPgammaNew;          
%---- Update of the conditional likelihood of gamma as well
          probprec = sum(VettMML) + sum(VettPgamma); %loglik_prop - Prior_prop;           
       else
       end
    else
    end
    tau2St(i) = tauprec2;                          
    
%-------------------- tau3 ---------------
    atap=0;
    while atap==0
        tauprop3 = normrnd(tauprec3,taupropsd);
        if tauprop3 > 0
            atap=1;
        end
    end
    if tauprop3 > 0        
       wprop = exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprop3*Starget1+tauprec4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprop3*Starget1+tauprec4*Starget2+tauprec5*Spita)); 
       loglik_prop = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprop3)-tauprop3/b_tau...
       +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;       
       for (iter = 1:(p*q))
         if(gamprec(iter)==1) 
            loglik_prop = loglik_prop + log(wprop(iter)); 
            VettPgammaNew(iter) = log(wprop(iter));
         else
            if wprop(iter)==1
                wprop(iter) = 1-10^-16;
            else
            end
            loglik_prop = loglik_prop + log(1.-wprop(iter)); 
            VettPgammaNew(iter) = log(1.-wprop(iter)); 
          end
       end        
       if(((loglik_prop+log(1-normcdf(0,tauprec3,taupropsd)))-(loglik+log(1-normcdf(0,tauprop3,taupropsd)))) > log(unifrnd(0,1,1,1)))
          tauprec3 = tauprop3;
          w = wprop;
          loglik = loglik_prop;
          accepttau3 = accepttau3 + 1;
          VettPgamma = VettPgammaNew;          
%---- Update of the conditional likelihood of gamma as well
          probprec = sum(VettMML) + sum(VettPgamma); 
       else
       end
    else
    end
    tau3St(i) = tauprec3; 
    
    
%-------------------- tau4 ---------------
    atap=0;
    while atap==0
        tauprop4 = normrnd(tauprec4,taupropsd);
        if tauprop4 > 0
            atap=1;
        end
    end
    if tauprop4 > 0        
       wprop = exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprop4*Starget2+tauprec5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprop4*Starget2+tauprec5*Spita)); 
       loglik_prop = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
       +(a_tau - 1)*log(tauprop4)-tauprop4/b_tau + (a_tau - 1)*log(tauprec5)-tauprec5/b_tau;       
       for (iter = 1:(p*q))
         if(gamprec(iter)==1) 
            loglik_prop = loglik_prop + log(wprop(iter)); 
            VettPgammaNew(iter) = log(wprop(iter));
         else
            if wprop(iter)==1
                wprop(iter) = 1-10^-16;
            else
            end
            loglik_prop = loglik_prop + log(1.-wprop(iter)); 
            VettPgammaNew(iter) = log(1.-wprop(iter)); 
          end
       end        
       if(((loglik_prop+log(1-normcdf(0,tauprec4,taupropsd)))-(loglik+log(1-normcdf(0,tauprop4,taupropsd)))) > log(unifrnd(0,1,1,1)))
          tauprec4 = tauprop4;
          w = wprop;
          loglik = loglik_prop;
          accepttau4 = accepttau4 + 1;
          VettPgamma = VettPgammaNew;          
%---- Update of the conditional likelihood of gamma as well
          probprec = sum(VettMML) + sum(VettPgamma); 
       else
       end
    else
    end
    tau4St(i) = tauprec4;
    
        
%-------------------- tau5 ---------------
    atap=0;
    while atap==0
        tauprop5 = normrnd(tauprec5,taupropsd);
        if tauprop5 > 0
            atap=1;
        end
    end
    if tauprop5 > 0        
       wprop = exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprop5*Spita)./(1.+exp(eta+tauprec*SPicTar+tauprec2*SMir+tauprec3*Starget1+tauprec4*Starget2+tauprop5*Spita)); 
       loglik_prop = (a_tau - 1)*log(tauprec) - tauprec/b_tau + (a_tau - 1)*log(tauprec2) - tauprec2/b_tau+ (a_tau - 1)*log(tauprec3)-tauprec3/b_tau...
       +(a_tau - 1)*log(tauprec4)-tauprec4/b_tau + (a_tau - 1)*log(tauprop5)-tauprop5/b_tau;       
       for (iter = 1:(p*q))
         if(gamprec(iter)==1) 
            loglik_prop = loglik_prop + log(wprop(iter));
            VettPgammaNew(iter) = log(wprop(iter));
         else
            if wprop(iter)==1
                wprop(iter) = 1-10^-16;
            else
            end
            loglik_prop = loglik_prop + log(1.-wprop(iter)); 
            VettPgammaNew(iter) = log(1.-wprop(iter)); 
          end
       end
       if(((loglik_prop+log(1-normcdf(0,tauprec5,taupropsd)))-(loglik+log(1-normcdf(0,tauprop5,taupropsd)))) > log(unifrnd(0,1,1,1)))
          tauprec5 = tauprop5;
          w = wprop;
          loglik = loglik_prop;
          accepttau5 = accepttau5 + 1;
          VettPgamma = VettPgammaNew;          
%---- Update of the conditional likelihood of gamma as well
          probprec = sum(VettMML) + sum(VettPgamma); 
       else
       end
    else
    end
    tau5St(i) = tauprec5;            
    
   
%------------ Update variances $\sigma_g$s    
    if i==1
        for j=1:q
            sig = sigVprec(j);   
            bsig = e/sig; asig = sig/bsig; 
		 snew = gamrnd(asig,bsig);
            signew = snew + cost;
            gammaYi=gamprec((((j-1)*p)+1):(j*p));
            ki=sum(gammaYi);
            a_sigNew = (a_sig+ki)/2;
            probSigProp = log((1/gamma(a_sigNew))*(b_sig^a_sigNew)*signew^(-a_sigNew-1)*exp(-b_sig/signew));
            probSigPrec = VettSig(j);
            bsig2 = e/signew; asig2 = signew/bsig2;
            prop = log(gampdf(sig-cost,asig2,bsig2)/gampdf(snew,asig,bsig));
            Yi = Y(:,j);      
            r = VettMML(j); 
            if ki>0
                Zi = X(:,find(gammaYi));
                Vi = inv(Zi'*Zi);
                Cinew = -Yi'*Zi-(sqrt(signew)/alph)*ones(1,ki);                
                rnew = -((nOss-ki)/2)*log(2*pi)-(nOss/2)*log(signew)-ki*log(alph)-(1/(2*signew))*(Yi'*Yi-Cinew*Vi*Cinew')+...
                    0.5*log(det(Vi))+log(mvncdf(zeros(ki,1),-Vi'*Cinew',signew.*Vi));                 
            else
                rnew = log(mvnpdf(Yi,zeros(nOss,1),diag(ones(nOss,1).*signew)));
            end                                       
            if (rnew+probSigProp-(r+probSigPrec)+prop > log(unifrnd(0,1,1,1)))            
                SigM(i,j) = signew;
                sigVprec(j) = signew;
                VettMML(j) = rnew; 
                VettSig(j) = probSigProp;            
                acceptsig = acceptsig + 1;
            else                           
                SigM(i,j) = sig;
            end
        end
    probprec = sum(VettMML) + sum(VettPgamma);    
    end
    ProbSt(i)=probprec+sum(VettSig);
end

SigMean = SigM;
