function [r,s,s2,ind] = mml_miRNAbn(gamnew,X,Y,elem,elem2,alph,w,sigVprec)

%mml_miRNAbn: log-marginal model likelihood. 
%*****************************************************************

%INPUTS:
% gamnew,		binary vector, (M*G) by 1
% X,			dependent variables (miRNAs), matrix (N by M)
% Y,            response variables (targets), matrix (N by G)
% elem,         position to add/delete
% elem2,		position used only for the swapping
% alph,         hyperparameter of the Gamma prior on the regression coefficients (hyperparameter $c$ in the manuscript)
% w,          	hyperparameter (Bernoulli priors)
% sigVprec,     vector of the sampled values $\sigma_g$s

%OUTPUTS:
% r,            log-relative posterior probability of gamma, element i
% s,            log-prior probability of r_{gm}, element i
% s2,           log-prior probability of r_{gm}, element j (only for the swapping)
% ind,          the element i (the gene)
%
%USAGE:
% [r,s,s2,ind] = mml_miRNAbn(gamnew,X,Y,elem,elem2,alph,w,sigVprec)

%NOTES:
% Called by bvsme_miRNA_Fbn and metro_miRNA_Fbn
%
%REFERENCE:
%*****************************************************************

[nOss p]=size(X);
ind=ceil(elem/p);

%------- log-marginal model likelihood of Yi
    
gammaYi=gamnew((((ind-1)*p)+1):(ind*p));
Yi=Y(:,ind);
ki=sum(gammaYi);  
sig = sigVprec(ind);
if ki>0        
    Zi=X(:,find(gammaYi));
    Ci = -Yi'*Zi-(sqrt(sig)/alph)*ones(1,ki);    
    Vi = inv(Zi'*Zi);       
    r = -((nOss-ki)/2)*log(2*pi)-(nOss/2)*log(sig)-ki*log(alph)-(1/(2*sig))*(Yi'*Yi-Ci*Vi*Ci')+...
            0.5*log(det(Vi))+log(mvncdf(zeros(ki,1),-Vi'*Ci',sig.*Vi)); 
else
    r = log(mvnpdf(Yi,zeros(nOss,1),diag(ones(nOss,1).*sig)));
end

             
%------ data integration prior on r_{gh}

if w(elem)==1
    w(elem)=1-10^-16;
    else
end

if w(elem2)==1
    w(elem2)=1-10^-16;
    else
end

s2=[];
if isempty(elem2)
    s = gamnew(elem)*log(w(elem))+(1-gamnew(elem))*log(1-w(elem));
else 
    s = gamnew(elem)*log(w(elem))+(1-gamnew(elem))*log(1-w(elem));
    s2= gamnew(elem2)*log(w(elem2))+(1-gamnew(elem2))*log(1-w(elem2));
end
